import logging
from logging import info
from urllib.parse import urljoin

import requests

from influx import BaseClient, Query


class InfluxQLClient(BaseClient):

    def __init__(self, host, port=8086, use_ssl=False, is_dns=False, user=None, password=None):
        """
        DB Client using Influx QL for interacting
        :param host: the host name
        :param port: the port to be used, defaults to 8086. Unused if is_dns is set to True
        :param use_ssl: indicates use SSL/TLS if True, False otherwise. Defaults to False
        :param is_dns: indicates if host represents DNS and not an IP/localhost. Defaults to False
        :param user: the user name to be used. Authentication is enabled if specified
        :param password: the password to be used in combination to user name. Used only if user name is specified
        """
        super().__init__(host, port, use_ssl=use_ssl, is_dns=is_dns, user=user, password=password)

    def create_db(self, name):
        """
        Creates a database with the given name
        :param name: the db name
        """
        query = f'CREATE DATABASE {name}'
        self.query(query)

    def get(self, query, db=None):
        """
        Executes GET query
        :param query: the query
        :param db: the db
        :return: response returned
        """
        url = urljoin(self._url, 'query')
        info(f'Invoking URL: {url}: Query: {query}')
        params = BaseClient.get_params(query, db)
        return requests.get(url, auth=(self._user, self._password), params=params)

    def write(self, db, measure, field, value, tags=None, timestamp=None):
        """
        Executes a write query to DB
        :param measure: the measure name
        :param field: the field key
        :param value: the field value
        :param db: the db name to query against
        :param tags: dict of tags to be saved against the point
        :param timestamp: the timestamp in nanoseconds from epoch time
        :return: response from server
        """
        return self.write_many_fields(db, measure, {field: value}, tags=tags, timestamp=timestamp)

    def write_many_fields(self, db, measure, value_by_field, tags=None, timestamp=None):
        """
        Executes a write query to DB
        :param measure: the measure name
        :param value_by_field: the value by field
        :param db: the db name to query against
        :param tags: dict of tags to be saved against the point
        :param timestamp: the timestamp in nanoseconds from epoch time
        :return: response from server
        """
        url = urljoin(self._url, 'write')
        tags = tags or dict()
        tags = ','.join(f'{tn}={tv}' for tn, tv in tags.items())
        fields = ','.join(f'{f}={v}' for f, v in value_by_field.items())
        data = f"{measure}{f',{tags}' if tags else ''} {fields} {timestamp or ''}".strip()
        info(f'Invoking URL: {url}: Query: {data}')
        return requests.post(url, auth=(self._user, self._password), data=data, params={'db': db})

    def fetch(self, db, query_inst: Query):
        """
        Gets all the rows matching the query instance
        :param db: the db name
        :param query_inst: the query instance
        :return: response as returned from the API
        """
        q = query_inst.build()
        return self.query(q, db=db)

    def fetch_all(self, db, measure):
        """
        Gets all the rows for the given measure
        :param db: the db name
        :param measure: the measure which you want to retrieve
        :return: response as returned from the API
        """
        q = Query().measure(measure).build()
        return self.query(q, db=db)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    client = InfluxQLClient('192.168.99.100')
    # info(client.create_db('test_db'))
    # write_resp = client.write('test_db', 'sample', 'val', 1.005, tags={'tag1': 'tag1Val', 'tag2': 'tag2Val'})
    # write_resp = client.write('test_db', 'sample', 'val', 1.005, tags={'tag1': 'tag3Val', 'tag2': 'tag4Val'})
    # write_resp = client.write_many_fields(
    #     'test_db',
    #     'sample', {
    #         'new_val': 1.007,
    #         'val': 1.01
    #     },
    #     tags={
    #         'tag1': 'tag3Val',
    #         'tag2': 'tag4Val',
    #     }
    # )
    # print(write_resp)
    # query = Query().measure('sample').select(['val', 'tag1', 'tag2']).where('tag1', 'tag1Val')
    # query = Query().measure('sample').select(['val', 'new_val', 'tag1', 'tag2'])
    # all_rows = client.fetch('test_db', query)
    query = Query().measure('price').select(['sym', 'inst', 'low', 'vol', 'prc']).where('quote', 'USDT').and_('base',
                                                                                                              'CRO')
    # query = Query().measure('price')
    all_rows = client.fetch('ta', query)
    # all_rows = client.fetch_all('ta', 'price')
    print(all_rows.json())



import logging

from influx import BaseClient


class InfluxAdminClient(BaseClient):

    def __init__(self, host, port=8086, use_ssl=False, is_dns=False, admin_user=None, admin_pwd=None):
        """
        Contains APIs for all the admin related query of user creation and privilege management.
        Requires an authenticated session from another user
        :param host: the host name
        :param port: the port to be used, defaults to 8086. Unused if is_dns is set to True
        :param use_ssl: indicates use SSL/TLS if True, False otherwise. Defaults to False
        :param is_dns: indicates if host represents DNS and not an IP/localhost. Defaults to False
        :param admin_user: the admin user to be used for authentication. Enables authentication.
        :param admin_pwd: the admin password to be used
        """
        super().__init__(host, port, use_ssl, is_dns, user=admin_user, password=admin_pwd)

    def create_user(self, user, password, is_admin=False):
        """
        Creates a new user in DB
        :param user: the user name
        :param password: the password for the user
        :param is_admin: created as an admin user if True, False otherwise. Defaults to False
        :return: response returned by the API
        """
        query = f"CREATE USER {user} WITH PASSWORD '{password}'{' WITH ALL PRIVILEGES' if is_admin else ''}"
        return self.query(query)

    def grant_all_privileges(self, user):
        """
        Grants all privileges to an existing user, elevating user to an admin user
        :param user: the user to be granted privileges
        :return: response returned by the API
        """
        query = f'GRANT ALL PRIVILEGES TO {user}'
        self.query(query)

    def revoke_all_privileges(self, user):
        """
        Revokes all privileges from an existing user
        :param user: the user to be revoked all privileges
        :return: response returned by the API
        """
        query = f'REVOKE ALL PRIVILEGES FROM {user}'
        self.query(query)

    def show_grants(self, user):
        """
        Shows all the grants existing for an user
        :param user: the user name
        :return: response from API
        """
        query = f'SHOW GRANTS FOR {user}'
        return self.query(query)

    def grant_privilege_on_db(self, db, user, privilege):
        """
        Grants specified privilege to user on DB
        :param db: the db name
        :param user: the user name
        :param privilege: the privilege to be provided
        :return: response from API
        """
        query = f'GRANT {privilege} ON {db} TO {user}'
        self.query(query)

    def _revoke_privilege(self, db, user, privilege):
        """
        Revokes specified privilege to user on DB
        :param db: the db name
        :param user: the user name
        :param privilege: the privilege to be revoked
        :return: response from API
        """
        query = f'REVOKE {privilege} ON {db} TO {user}'
        self.query(query)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    a_client = InfluxAdminClient('192.168.99.100', admin_user='admin', admin_pwd='admin')
    response = a_client.create_user('nkm', 'nkm', is_admin=True)
    print(response.status_code)
    print(response.json())



import os
from logging import info
from urllib.parse import urljoin

import requests


class BaseClient(object):

    def __init__(self, host, port=8086, use_ssl=False, is_dns=False, user=None, password=None):
        """
        Authenticates and creates a session to be used for DB operations
        :param host: the host name
        :param port: the port to be used, defaults to 8086. Unused if is_dns is set to True
        :param use_ssl: indicates use SSL/TLS if True, False otherwise. Defaults to False
        :param is_dns: indicates if host represents DNS and not an IP/localhost. Defaults to False
        :param user: the user name to be used. Authentication is enabled if specified
        :param password: the password to be used in combination to user name. Used only if user name is specified
        """
        scheme = 'https' if use_ssl else 'http'
        host = host if is_dns else f'{host}:{port}'
        self._url = f'{scheme}://{host}'
        self._user = user or os.environ.get('INFLUX_DB_USER')
        self._password = password or os.environ.get('INFLUX_DB_PASSWORD')
        info(f'Client configured with base URL: {self._url}')

    def query(self, query, db=None):
        """
        Executes the query provided
        :param query: the query to be executed
        :param db: the db name to query against
        :return: response returned
        """
        url = urljoin(self._url, 'query')
        info(f'Invoking URL: {url}: Query: {query}')
        params = BaseClient.get_params(query, db)
        return requests.post(url, auth=(self._user, self._password), params=params)

    @staticmethod
    def get_params(query, db):
        params = {'q': query}
        params.update({'db': db} if db else {})
        return params


class Query(object):

    def __init__(self):
        self._select = list()
        self._where = list()
        self._measure = None

    def select(self, cols):
        """
        Columns to be part of the select clause
        :param cols: list of columns
        """
        self._select.extend(cols)
        return self

    def measure(self, name):
        """ The measure name """
        self._measure = name
        return self

    def where(self, col, val):
        """
        The where clause for the query
        :param col: the column name
        :param val: the value to filter
        """
        self._where.append((None, col, val))
        return self

    def and_(self, col, val):
        """
        AND condition to be appended
        :param col: the column name
        :param val: the value
        """
        assert self._where, "Cannot add operation without WHERE condition"
        self._where.append(('AND', col, val))
        return self

    def or_(self, col, val):
        """
        OR condition to be appended
        :param col: the column name
        :param val: the value
        """
        assert self._where, "Cannot add operation without WHERE condition"
        self._where.append(('OR', col, val))
        return self

    def build(self):
        """ Builds the final Influx QL query """
        is_project_specific_cols = bool(self._select)
        select = '","'.join(self._select) if is_project_specific_cols else '*'
        select = f'"{select}"' if is_project_specific_cols else select
        where = ' '.join(
            [f'"{c}"=\'{v}\'' if i == 0 else f'{o} "{c}"=\'{v}\'' for i, (o, c, v) in enumerate(self._where)])
        where = f'WHERE {where}' if where else ''
        return f'SELECT {select} FROM "{self._measure}" {where}'



# -*- coding: utf-8 -*-
from distutils.core import setup

from setuptools import find_packages

setup(
    name='py-influxdb',
    packages=find_packages(exclude=('tests',)),
    version='2020.09.05.1',
    description='Python Client for Influx DB',
    long_description='Leverages the REST APIs exposed by Influx DB Server to write/query data',
    long_description_content_type="text/markdown",
    author='Nikhil K Madhusudhan (nikhilkmdev)',
    author_email='nikhilkmdev@gmail.com',
    maintainer='Nikhil K Madhusudhan (nikhilkmdev)',
    maintainer_email='nikhilkmdev@gmail.com',
    install_requires=['requests==2.24.0', 'py-influxdb'],
    keywords=['influxdb', 'influx-db', 'influx', 'timeseries', 'python3'],
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
)
